from fastapi import APIRouter, Depends, HTTPException
from sqlmodel import Session, select, delete
from geopy.geocoders import Nominatim
from geopy.distance import geodesic
from backend.database import get_session
from backend.routes.auth import get_current_user
from backend.models.models import (
    Order,
    OrderItem,
    User,
    Product,
    Shop,
    Payment,
    Cart,
    CartItem,
)
from backend.schemas.order import OrderCreate, OrderRead, OrderUpdate, CartItemCreate
from typing import Optional

router = APIRouter()


@router.post("/cart/add")
def add_to_cart(
    cart_item: CartItemCreate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Validate product exists and has enough stock
    product = session.get(Product, cart_item.product_id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")

    if product.stock < cart_item.quantity:
        raise HTTPException(status_code=400, detail="Not enough stock available")

    # Verify the shop exists
    shop = session.get(Shop, cart_item.shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")

    # Get or create a cart for the user
    cart = session.exec(select(Cart).where(Cart.user_id == current_user.id)).first()

    if not cart:
        cart = Cart(user_id=current_user.id)
        session.add(cart)
        session.commit()
        session.refresh(cart)

    # Check if the item already exists in the cart
    existing_item = session.exec(
        select(CartItem).where(
            CartItem.cart_id == cart.id, CartItem.product_id == cart_item.product_id
        )
    ).first()

    if existing_item:
        # Update quantity if item already exists
        existing_item.quantity += cart_item.quantity
        session.add(existing_item)
    else:
        # Create new cart item
        new_cart_item = CartItem(
            cart_id=cart.id,
            product_id=cart_item.product_id,
            quantity=cart_item.quantity,
            price=product.price,
        )
        session.add(new_cart_item)

    session.commit()

    return {"message": "Item added to cart successfully"}


@router.get("/cart/view-items")
def get_cart_items(
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Get user's cart
    cart = session.exec(select(Cart).where(Cart.user_id == current_user.id)).first()

    if not cart:
        return {"items": []}

    # Get cart items with product details
    cart_items = session.exec(select(CartItem).where(CartItem.cart_id == cart.id)).all()

    items_with_details = []
    for item in cart_items:
        product = session.get(Product, item.product_id)
        if product:
            product_dict = {
                "id": product.id,
                "name": product.name,
                "price": product.price,
                "shop_id": product.shop_id,
                "images": product.images,
                "quantity": item.quantity,
            }
            items_with_details.append(product_dict)

    return {"items": items_with_details}


@router.post("/cart/checkout")
def checkout_cart(
    checkout_data: dict,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Extract parameters from request body
    delivery_address = checkout_data.get("delivery_address")
    payment_id = checkout_data.get("payment_id")

    if not delivery_address or not payment_id:
        raise HTTPException(
            status_code=400,
            detail="Missing required parameters: delivery_address and payment_id",
        )

    # Get user's cart
    cart = session.exec(select(Cart).where(Cart.user_id == current_user.id)).first()

    if not cart:
        raise HTTPException(status_code=404, detail="Cart not found")

    # Get cart items
    cart_items = session.exec(select(CartItem).where(CartItem.cart_id == cart.id)).all()

    if not cart_items:
        raise HTTPException(status_code=400, detail="Cart is empty")

    # Validate payment method
    payment = session.get(Payment, payment_id)
    if not payment or payment.user_id != current_user.id:
        raise HTTPException(
            status_code=400, detail="Invalid or unauthorized payment method"
        )

    # Group cart items by shop
    shop_items = {}
    for item in cart_items:
        product = session.get(Product, item.product_id)
        if not product or product.stock < item.quantity:
            raise HTTPException(
                status_code=400, detail=f"Product {item.product_id} is out of stock"
            )

        if product.shop_id not in shop_items:
            shop_items[product.shop_id] = []

        shop_items[product.shop_id].append(item)

    # Create orders for each shop
    orders = []
    for shop_id, items in shop_items.items():
        shop = session.get(Shop, shop_id)

        # Geocode the delivery address
        geolocator = Nominatim(user_agent="order_locator")
        try:
            delivery_location = geolocator.geocode(delivery_address)
            if not delivery_location:
                # Fall back to London coordinates if geocoding fails
                delivery_location = type(
                    "obj", (object,), {"latitude": 51.5074, "longitude": -0.1278}
                )
        except Exception as e:
            print(f"Geocoding error: {e}")
            # Fall back to London coordinates
            delivery_location = type(
                "obj", (object,), {"latitude": 51.5074, "longitude": -0.1278}
            )

        # Calculate the distance between the shop and the delivery location
        shop_location = (shop.latitude, shop.longitude)
        delivery_coordinates = (delivery_location.latitude, delivery_location.longitude)
        distance_km = geodesic(shop_location, delivery_coordinates).kilometers

        # Calculate the shipping price - Tiered approach
        # Base fee + distance fee, capped at maximum
        base_shipping_fee = 5.99
        distance_fee_per_km = 0.10
        max_shipping_fee = 14.99

        # Calculate shipping with distance component
        calculated_shipping = base_shipping_fee + (distance_km * distance_fee_per_km)

        # Cap at maximum fee
        shipping_price = min(calculated_shipping, max_shipping_fee)

        # Calculate the total price
        product_total = sum(item.price * item.quantity for item in items)
        total_price = shipping_price + product_total

        # Create the order
        new_order = Order(
            user_id=current_user.id,
            shop_id=shop_id,
            payment_id=payment_id,
            total_price=total_price,
            shipping_price=shipping_price,
            status="pending",
            delivery_address=delivery_address,
            delivery_latitude=delivery_location.latitude,
            delivery_longitude=delivery_location.longitude,
        )
        session.add(new_order)
        session.commit()
        session.refresh(new_order)

        # Create order items and update product stock
        for item in items:
            order_item = OrderItem(
                order_id=new_order.id,
                product_id=item.product_id,
                quantity=item.quantity,
                price=item.price,
            )
            session.add(order_item)

            # Update product stock
            product = session.get(Product, item.product_id)
            product.stock -= item.quantity
            session.add(product)

        orders.append(new_order.id)

    # Clear the cart after checkout
    session.exec(delete(CartItem).where(CartItem.cart_id == cart.id))
    session.commit()

    return {"message": "Orders created successfully", "order_ids": orders}


@router.delete("/cart/remove/{product_id}")
def remove_from_cart(
    product_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Get user's cart
    cart = session.exec(select(Cart).where(Cart.user_id == current_user.id)).first()

    if not cart:
        raise HTTPException(status_code=404, detail="Cart not found")

    # Find the cart item
    cart_item = session.exec(
        select(CartItem).where(
            CartItem.cart_id == cart.id, CartItem.product_id == product_id
        )
    ).first()

    if not cart_item:
        raise HTTPException(status_code=404, detail="Item not found in cart")

    # Remove the item
    session.delete(cart_item)
    session.commit()

    return {"message": "Item removed from cart successfully"}


@router.post("/add")
def legacy_add_to_cart(
    cart_item: CartItemCreate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # For backward compatibility, redirect to new cart endpoint
    return add_to_cart(cart_item, session, current_user)


@router.post("/", response_model=OrderRead)
def create_order(
    order_data: OrderCreate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Fetch the shop details
    shop = session.get(Shop, order_data.shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")

    # Validate the payment ID
    payment = session.get(Payment, order_data.payment_id)
    if not payment or payment.user_id != current_user.id:
        raise HTTPException(
            status_code=400, detail="Invalid or unauthorized payment method"
        )

    # Geocode the delivery address
    geolocator = Nominatim(user_agent="order_locator")
    delivery_location = geolocator.geocode(order_data.delivery_address)
    if not delivery_location:
        raise HTTPException(status_code=400, detail="Invalid delivery address provided")

    # Calculate the distance between the shop and the delivery location
    shop_location = (shop.latitude, shop.longitude)
    delivery_coordinates = (delivery_location.latitude, delivery_location.longitude)
    distance_km = geodesic(shop_location, delivery_coordinates).kilometers

    # Calculate the shipping price - Tiered approach
    # Base fee + distance fee, capped at maximum
    base_shipping_fee = 5.99
    distance_fee_per_km = 0.10
    max_shipping_fee = 14.99

    # Calculate shipping with distance component
    calculated_shipping = base_shipping_fee + (distance_km * distance_fee_per_km)

    # Cap at maximum fee
    shipping_price = min(calculated_shipping, max_shipping_fee)

    total_price = shipping_price
    for item in order_data.items:
        product = session.get(Product, item.product_id)
        if not product or product.stock < item.quantity:
            raise HTTPException(
                status_code=400, detail=f"Product {item.product_id} is out of stock"
            )
        total_price += item.quantity * product.price

    # Create the order
    new_order = Order(
        user_id=current_user.id,
        shop_id=order_data.shop_id,
        payment_id=order_data.payment_id,
        total_price=total_price,
        shipping_price=shipping_price,
        status="pending",
        delivery_address=order_data.delivery_address,
        delivery_latitude=delivery_location.latitude,
        delivery_longitude=delivery_location.longitude,
    )
    session.add(new_order)
    session.commit()
    session.refresh(new_order)

    # Create order items
    for item in order_data.items:
        order_item = OrderItem(
            order_id=new_order.id,
            product_id=item.product_id,
            quantity=item.quantity,
            price=product.price,
        )
        session.add(order_item)
        product.stock -= item.quantity

    session.commit()
    return new_order


# list all orders associated with the current user
@router.get("/list", response_model=list[OrderRead])
def list_orders(
    status: Optional[str] = None,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    query = select(Order).where(Order.user_id == current_user.id)

    # Filter by status if provided
    if status:
        query = query.where(Order.status == status)

    orders = session.exec(query).all()
    return orders


# retrieve orders for a specific shop (shop owner only)
@router.get("/shop/{shop_id}", response_model=list[OrderRead])
def get_shop_orders(
    shop_id: int,
    status: Optional[str] = None,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Verify the shop belongs to the current user
    shop = session.get(Shop, shop_id)
    if not shop or shop.owner_id != current_user.id:
        raise HTTPException(
            status_code=403, detail="Unauthorized to access this shop's orders"
        )

    # Query orders for this shop
    query = select(Order).where(Order.shop_id == shop_id)

    # Filter by status if provided
    if status:
        query = query.where(Order.status == status)

    orders = session.exec(query).all()
    return orders


# retrieve a specific order, both buyer and shop owner can access
@router.get("/{order_id}", response_model=OrderRead)
def get_order(
    order_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order:
        raise HTTPException(status_code=404, detail="Order not found")

    # Allow access if user is the buyer or the shop owner
    shop = session.get(Shop, order.shop_id)
    if order.user_id != current_user.id and (
        not shop or shop.owner_id != current_user.id
    ):
        raise HTTPException(status_code=403, detail="Unauthorized to access this order")

    return order


# update order status (more flexible for both user and shop owner)
@router.put("/update_status/{order_id}", response_model=OrderRead)
def update_order_status(
    order_id: int,
    data: dict,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order:
        raise HTTPException(status_code=404, detail="Order not found")

    # Check permissions
    is_buyer = order.user_id == current_user.id
    shop = session.get(Shop, order.shop_id)
    is_shop_owner = shop and shop.owner_id == current_user.id

    if not is_buyer and not is_shop_owner:
        raise HTTPException(status_code=403, detail="Unauthorized to update this order")

    # Get the new status
    new_status = data.get("status")
    if not new_status:
        raise HTTPException(status_code=400, detail="Status is required")

    # Validate status transitions
    valid_transitions = {
        # Current status: [allowed next statuses]
        "pending": ["processing", "shipped", "cancelled"],
        "processing": ["shipped", "cancelled"],
        "shipped": ["delivered", "cancelled"],
        "delivered": ["completed"],
        "completed": [],  # Final state
        "cancelled": [],  # Final state
    }

    if new_status not in valid_transitions.get(order.status, []):
        raise HTTPException(
            status_code=400,
            detail=f"Invalid status transition from '{order.status}' to '{new_status}'",
        )

    # Check who can make this transition
    if is_shop_owner and new_status in ["shipped", "processing", "cancelled"]:
        # Shop owner can ship, process or cancel orders
        order.status = new_status
    elif is_buyer and new_status in ["delivered", "cancelled"]:
        # Buyers can mark as delivered or cancel their orders
        order.status = new_status
    else:
        raise HTTPException(
            status_code=403,
            detail=f"You are not authorized to change the order status to '{new_status}'",
        )

    # Update the order
    session.add(order)
    session.commit()
    session.refresh(order)
    return order


# Update order details (keep the original function with modifications)
@router.put("/{order_id}", response_model=OrderRead)
def update_order(
    order_id: int,
    order_update: OrderUpdate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order or order.user_id != current_user.id:
        raise HTTPException(status_code=404, detail="Order not found or unauthorized")

    # Update the delivery address if provided
    if order_update.delivery_address:
        geolocator = Nominatim(user_agent="order_locator")
        delivery_location = geolocator.geocode(order_update.delivery_address)
        if not delivery_location:
            raise HTTPException(status_code=400, detail="Invalid delivery address")
        order.delivery_address = order_update.delivery_address
        order.delivery_latitude = delivery_location.latitude
        order.delivery_longitude = delivery_location.longitude

        # Recalculate the shipping price
        shop = session.get(Shop, order.shop_id)
        if not shop:
            raise HTTPException(status_code=404, detail="Shop not found")
        shop_location = (shop.latitude, shop.longitude)
        delivery_coordinates = (delivery_location.latitude, delivery_location.longitude)
        distance_km = geodesic(shop_location, delivery_coordinates).kilometers

        # Calculate the shipping price - Tiered approach
        # Base fee + distance fee, capped at maximum
        base_shipping_fee = 5.99
        distance_fee_per_km = 0.10
        max_shipping_fee = 14.99

        # Calculate shipping with distance component
        calculated_shipping = base_shipping_fee + (distance_km * distance_fee_per_km)

        # Cap at maximum fee
        shipping_price = min(calculated_shipping, max_shipping_fee)

        # Update the total price
        product_total = sum(
            item.quantity * session.get(Product, item.product_id).price
            for item in order.order_items
        )
        order.shipping_price = shipping_price
        order.total_price = product_total + shipping_price

    # Update the order status if provided
    if order_update.status:
        if order.status != "pending" or order_update.status != "completed":
            raise HTTPException(
                status_code=400,
                detail="Order can only be updated from pending to completed",
            )
        order.status = order_update.status

    session.commit()
    session.refresh(order)
    return order


# Delete order
@router.delete("/{order_id}")
def delete_order(
    order_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order or (
        order.user_id != current_user.id and current_user.role != "shop_owner"
    ):
        raise HTTPException(status_code=404, detail="Order not found or unauthorized")

    # Delete related order items first
    session.exec(delete(OrderItem).where(OrderItem.order_id == order.id))

    # Now delete the order
    session.delete(order)
    session.commit()

    return {"message": "Order deleted successfully"}
