from pydantic import BaseModel, Field, validator
from datetime import datetime
from backend.utils.security import (
    encrypt_card_number,
    decrypt_card_number,
)


class PaymentCreate(BaseModel):
    payment_method: str = Field(..., min_length=3, max_length=50)
    card_number: str = Field(..., min_length=12, max_length=19, pattern=r"^\d+$")
    cvv: str = Field(..., min_length=3, max_length=3, pattern=r"^\d+$")
    expiry_date: str  # Expecting MM/YY format

    @validator("card_number")
    def encrypt_card(cls, value):
        """Encrypt card number before storing."""
        return encrypt_card_number(value)

    @validator("cvv")
    def encrypt_cvv(cls, value):
        """Encrypt CVV before storing."""
        return encrypt_card_number(value)

    @validator("expiry_date")
    def validate_expiry_date(cls, value):
        """Validate expiry date format (MM/YY) and ensure it's in the future."""
        try:
            expiry_obj = datetime.strptime(value, "%m/%y")
            current_date = datetime.now()

            if expiry_obj.year < current_date.year or (
                expiry_obj.year == current_date.year
                and expiry_obj.month < current_date.month
            ):
                raise ValueError("Card expiry date must be in the future")

            return value
        except ValueError:
            raise ValueError("Invalid expiry date format. Use MM/YY")


class PaymentRead(BaseModel):
    id: int
    user_id: int
    payment_method: str
    card_number: str  # Now storing the actual card number
    expiry_date: str
    created_at: datetime

    @classmethod
    def from_orm(cls, obj):
        """Decrypt card number before returning."""
        decrypted_card = decrypt_card_number(obj.card_number)
        return cls(
            id=obj.id,
            user_id=obj.user_id,
            payment_method=obj.payment_method,
            card_number=decrypted_card,  # Return actual card number
            expiry_date=obj.expiry_date,
            created_at=obj.created_at,
        )

    class Config:
        from_attributes = True  # Enable ORM mode
