import customtkinter as ctk
import requests
from tkinter import messagebox
from PIL import Image, ImageTk
import io
from datetime import datetime

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
SIDEBAR_BG = "#2D3035"


def owner_orders_frame(parent, switch_func, API_URL, token):
    """
    Order management page for shop owners.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token  # Add method to the frame

    # Create layout: sidebar + main content
    sidebar_frame = ctk.CTkFrame(frame, fg_color=SIDEBAR_BG, width=180)
    sidebar_frame.pack(side="left", fill="y")
    sidebar_frame.pack_propagate(False)  # Don't shrink

    main_frame = ctk.CTkFrame(frame, fg_color=DARK_BG)
    main_frame.pack(side="left", fill="both", expand=True)

    # --- SIDEBAR CONTENT ---
    sidebar_title = ctk.CTkLabel(
        sidebar_frame,
        text="Shop\nDashboard",
        font=("Helvetica", 18, "bold"),
        text_color="white",
    )
    sidebar_title.pack(pady=(20, 30))

    def create_sidebar_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            command=command,
            fg_color=SHOPPING if is_active else "transparent",
            text_color="white",
            hover_color="#3b3b3b" if not is_active else None,
            height=40,
            anchor="w",
            font=("Helvetica", 14),
        )

    # Button for revenue dashboard
    def open_revenue_dashboard():
        switch_func("owner_dashboard")

    revenue_btn = create_sidebar_button("Revenue", open_revenue_dashboard)
    revenue_btn.pack(fill="x", pady=5, padx=10)

    # Button for product management
    def open_product_management():
        switch_func("owner_products")

    product_btn = create_sidebar_button("Product Management", open_product_management)
    product_btn.pack(fill="x", pady=5, padx=10)

    # Active button for order management
    orders_btn = create_sidebar_button("Order Management", is_active=True)
    orders_btn.pack(fill="x", pady=5, padx=10)

    # Button to go to buyer dashboard
    def go_to_shop():
        # Use the dashboard_bypass flag to avoid role-based routing
        switch_func("dashboard_bypass", token)

    shop_btn = create_sidebar_button("Go to Shop", go_to_shop)
    shop_btn.pack(fill="x", pady=5, padx=10)

    # Logout button at bottom of sidebar
    def logout():
        switch_func("login")

    logout_btn = ctk.CTkButton(
        sidebar_frame,
        text="Logout",
        command=logout,
        fg_color="#555",
        text_color="white",
        hover_color="#777",
        height=40,
    )
    logout_btn.pack(side="bottom", fill="x", pady=20, padx=10)

    # --- MAIN CONTENT ---
    # Title
    title_label = ctk.CTkLabel(
        main_frame,
        text="Order Management",
        font=("Helvetica", 24, "bold"),
        text_color="white",
    )
    title_label.pack(anchor="w", padx=30, pady=(30, 20))

    # Tab buttons for Pending/Shipped orders
    tabs_frame = ctk.CTkFrame(main_frame, fg_color="transparent")
    tabs_frame.pack(fill="x", padx=30, pady=(0, 20))

    active_tab = ctk.StringVar(value="pending")

    def switch_tab(tab_name):
        active_tab.set(tab_name)
        if tab_name == "pending":
            pending_btn.configure(fg_color=SHOPPING)
            shipped_btn.configure(fg_color="#555")
            pending_orders_frame.pack(fill="both", expand=True)
            shipped_orders_frame.pack_forget()
            fetch_pending_orders()
        else:
            pending_btn.configure(fg_color="#555")
            shipped_btn.configure(fg_color=SHOPPING)
            pending_orders_frame.pack_forget()
            shipped_orders_frame.pack(fill="both", expand=True)
            fetch_shipped_orders()

    pending_btn = ctk.CTkButton(
        tabs_frame,
        text="Pending Orders",
        command=lambda: switch_tab("pending"),
        fg_color=SHOPPING,  # Initially active
        hover_color="#0096ff",
        height=40,
        width=150,
    )
    pending_btn.pack(side="left", padx=(0, 10))

    shipped_btn = ctk.CTkButton(
        tabs_frame,
        text="Shipped Orders",
        command=lambda: switch_tab("shipped"),
        fg_color="#555",
        hover_color="#777",
        height=40,
        width=150,
    )
    shipped_btn.pack(side="left")

    # Container frames for each tab
    pending_orders_frame = ctk.CTkScrollableFrame(main_frame, fg_color="transparent")
    pending_orders_frame.pack(fill="both", expand=True, padx=30, pady=(0, 20))

    shipped_orders_frame = ctk.CTkScrollableFrame(main_frame, fg_color="transparent")
    # Not packed initially - will be shown when tab is switched

    # Fix scrolling behavior
    def _bound_to_mousewheel(event, frame):
        frame._parent_canvas.bind_all(
            "<MouseWheel>", lambda e: _on_mousewheel(e, frame)
        )

    def _unbound_to_mousewheel(event, frame):
        frame._parent_canvas.unbind_all("<MouseWheel>")

    def _on_mousewheel(event, frame):
        frame._parent_canvas.yview_scroll(int(-1 * (event.delta / 120)), "units")

    # Bind scrolling to both frames
    pending_orders_frame.bind(
        "<Enter>", lambda e: _bound_to_mousewheel(e, pending_orders_frame)
    )
    pending_orders_frame.bind(
        "<Leave>", lambda e: _unbound_to_mousewheel(e, pending_orders_frame)
    )
    shipped_orders_frame.bind(
        "<Enter>", lambda e: _bound_to_mousewheel(e, shipped_orders_frame)
    )
    shipped_orders_frame.bind(
        "<Leave>", lambda e: _unbound_to_mousewheel(e, shipped_orders_frame)
    )

    # Store order cards
    pending_order_cards = []
    shipped_order_cards = []

    def clear_order_cards(order_list):
        """Clear all order cards from the specified list"""
        for card in order_list:
            card.destroy()
        order_list.clear()

    def create_order_card(parent, order_data, is_pending=True):
        """Create an order card with order details and action buttons"""
        # Create a card frame
        card = ctk.CTkFrame(parent, fg_color=CARD_BG, corner_radius=10, height=180)
        card.pack(fill="x", pady=10, padx=5)
        card.pack_propagate(False)

        # Left section - Order info
        info_frame = ctk.CTkFrame(card, fg_color="transparent", width=400)
        info_frame.pack(side="left", fill="y", padx=15, pady=15)
        info_frame.pack_propagate(False)

        # Order ID and date
        order_id = order_data.get("id", "N/A")
        order_date = order_data.get("created_at", "")
        try:
            # Convert ISO format to readable date
            formatted_date = datetime.fromisoformat(
                order_date.replace("Z", "+00:00")
            ).strftime("%B %d, %Y")
        except:
            formatted_date = order_date

        order_label = ctk.CTkLabel(
            info_frame,
            text=f"Order #{order_id}",
            font=("Helvetica", 16, "bold"),
            text_color="white",
        )
        order_label.pack(anchor="w")

        date_label = ctk.CTkLabel(
            info_frame,
            text=f"Placed on {formatted_date}",
            font=("Helvetica", 12),
            text_color="#999",
        )
        date_label.pack(anchor="w", pady=(0, 10))

        # Customer info
        customer = order_data.get("buyer", {})
        customer_name = customer.get("username", "Unknown")

        customer_label = ctk.CTkLabel(
            info_frame,
            text=f"Customer: {customer_name}",
            font=("Helvetica", 14),
            text_color="white",
        )
        customer_label.pack(anchor="w", pady=(0, 5))

        # Total amount
        total_amount = sum(
            item.get("price", 0) * item.get("quantity", 0)
            for item in order_data.get("order_items", [])
        )

        amount_label = ctk.CTkLabel(
            info_frame,
            text=f"Total: ${total_amount:.2f}",
            font=("Helvetica", 14, "bold"),
            text_color=SHOPPING,
        )
        amount_label.pack(anchor="w")

        # Right section - Products and actions
        products_frame = ctk.CTkFrame(card, fg_color="transparent")
        products_frame.pack(
            side="left", fill="both", expand=True, padx=(0, 15), pady=15
        )

        # Product count
        items = order_data.get("order_items", [])
        product_count_label = ctk.CTkLabel(
            products_frame,
            text=f"{len(items)} Product(s)",
            font=("Helvetica", 14, "bold"),
            text_color="white",
        )
        product_count_label.pack(anchor="w", pady=(0, 5))

        # List of products (limited to first 2 for space)
        products_list = ctk.CTkFrame(products_frame, fg_color="transparent")
        products_list.pack(fill="x", expand=True)

        for i, item in enumerate(items[:2]):
            product_name = item.get("product", {}).get("name", "Unknown Product")
            quantity = item.get("quantity", 0)

            item_label = ctk.CTkLabel(
                products_list,
                text=f"{product_name} (x{quantity})",
                font=("Helvetica", 12),
                text_color="#ccc",
            )
            item_label.pack(anchor="w")

        if len(items) > 2:
            more_label = ctk.CTkLabel(
                products_list,
                text=f"... and {len(items) - 2} more",
                font=("Helvetica", 12, "italic"),
                text_color="#999",
            )
            more_label.pack(anchor="w")

        # Action button for pending orders
        if is_pending:

            def confirm_shipment():
                if messagebox.askyesno(
                    "Confirm Shipment",
                    f"Confirm that order #{order_id} has been shipped?",
                ):
                    update_order_status(order_id, "shipped")

            confirm_btn = ctk.CTkButton(
                products_frame,
                text="Confirm Shipment",
                command=confirm_shipment,
                fg_color=SHOPPING,
                hover_color="#0096ff",
                height=30,
            )
            confirm_btn.pack(side="bottom", fill="x")
        else:
            # For shipped orders, show shipped status
            status_frame = ctk.CTkFrame(
                products_frame, fg_color="#1a5e2d", height=30, corner_radius=5
            )
            status_frame.pack(side="bottom", fill="x")
            status_frame.pack_propagate(False)

            status_label = ctk.CTkLabel(
                status_frame,
                text="✓ Shipped",
                font=("Helvetica", 12, "bold"),
                text_color="#8be0a6",
            )
            status_label.place(relx=0.5, rely=0.5, anchor="center")

        return card

    def update_order_status(order_id, new_status):
        """Update an order's status via API call"""
        headers = {"Authorization": f"Bearer {token}"}
        try:
            data = {"status": new_status}
            resp = requests.put(
                f"{API_URL}/order/update_status/{order_id}", json=data, headers=headers
            )

            if resp.status_code == 200:
                messagebox.showinfo(
                    "Success", f"Order #{order_id} marked as {new_status}"
                )
                # Refresh both tabs
                fetch_pending_orders()
                fetch_shipped_orders()
            else:
                messagebox.showerror("Error", f"Failed to update order: {resp.text}")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    def fetch_pending_orders():
        """Fetch pending orders for the shop owner's products"""
        headers = {"Authorization": f"Bearer {token}"}
        try:
            # First get the owner's shop
            shop_resp = requests.get(f"{API_URL}/shops/owner", headers=headers)
            if shop_resp.status_code == 200:
                shops = shop_resp.json()
                if not shops:
                    # No shops found
                    clear_order_cards(pending_order_cards)
                    no_shop_label = ctk.CTkLabel(
                        pending_orders_frame,
                        text="You don't have any shops",
                        font=("Helvetica", 14),
                        text_color="gray",
                    )
                    pending_order_cards.append(no_shop_label)
                    no_shop_label.pack(pady=30)
                    return

                shop_id = shops[0]["id"]  # Use the first shop

                # Get pending orders for this shop
                resp = requests.get(
                    f"{API_URL}/order/shop/{shop_id}?status=pending", headers=headers
                )

                if resp.status_code == 200:
                    orders = resp.json()

                    # Clear existing cards
                    clear_order_cards(pending_order_cards)

                    if not orders:
                        no_orders_label = ctk.CTkLabel(
                            pending_orders_frame,
                            text="No pending orders found",
                            font=("Helvetica", 14),
                            text_color="gray",
                        )
                        pending_order_cards.append(no_orders_label)
                        no_orders_label.pack(pady=30)
                        return

                    # Create cards for each order
                    for order in orders:
                        card = create_order_card(
                            pending_orders_frame, order, is_pending=True
                        )
                        pending_order_cards.append(card)
                else:
                    messagebox.showerror(
                        "Error", f"Failed to fetch pending orders: {resp.text}"
                    )
            else:
                messagebox.showerror("Error", f"Failed to fetch shop: {shop_resp.text}")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    def fetch_shipped_orders():
        """Fetch shipped orders for the shop owner's products"""
        headers = {"Authorization": f"Bearer {token}"}
        try:
            # First get the owner's shop
            shop_resp = requests.get(f"{API_URL}/shops/owner", headers=headers)
            if shop_resp.status_code == 200:
                shops = shop_resp.json()
                if not shops:
                    # No shops found
                    clear_order_cards(shipped_order_cards)
                    no_shop_label = ctk.CTkLabel(
                        shipped_orders_frame,
                        text="You don't have any shops",
                        font=("Helvetica", 14),
                        text_color="gray",
                    )
                    shipped_order_cards.append(no_shop_label)
                    no_shop_label.pack(pady=30)
                    return

                shop_id = shops[0]["id"]  # Use the first shop

                # Get shipped orders for this shop
                resp = requests.get(
                    f"{API_URL}/order/shop/{shop_id}?status=shipped", headers=headers
                )

                if resp.status_code == 200:
                    orders = resp.json()

                    # Clear existing cards
                    clear_order_cards(shipped_order_cards)

                    if not orders:
                        no_orders_label = ctk.CTkLabel(
                            shipped_orders_frame,
                            text="No shipped orders found",
                            font=("Helvetica", 14),
                            text_color="gray",
                        )
                        shipped_order_cards.append(no_orders_label)
                        no_orders_label.pack(pady=30)
                        return

                    # Create cards for each order
                    for order in orders:
                        card = create_order_card(
                            shipped_orders_frame, order, is_pending=False
                        )
                        shipped_order_cards.append(card)
                else:
                    messagebox.showerror(
                        "Error", f"Failed to fetch shipped orders: {resp.text}"
                    )
            else:
                messagebox.showerror("Error", f"Failed to fetch shop: {shop_resp.text}")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    # Initialize by fetching pending orders
    fetch_pending_orders()

    # Add refresh method to frame for external calls
    def refresh_data(*args):
        if active_tab.get() == "pending":
            fetch_pending_orders()
        else:
            fetch_shipped_orders()

    frame.refresh_data = refresh_data

    return frame
