import customtkinter as ctk
from tkinter import filedialog, messagebox
import requests
import os
from PIL import Image, ImageTk
import io

# Color scheme matching app theme
SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
SIDEBAR_BG = "#2D3035"


def edit_product_frame(parent, switch_func, API_URL, token, product_id=None):
    """
    Frame to edit an existing product.
    Parameters:
    - parent: The parent widget
    - switch_func: Function to switch between frames
    - API_URL: API base URL
    - token: User authentication token
    - product_id: ID of the product to edit
    """
    # Main container with dark theme
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token  # Add method to the frame

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar, text="Edit Product", text_color="white", font=("Helvetica", 20, "bold")
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        switch_func("owner_products")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14),
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN CONTENT ---
    main_content = ctk.CTkFrame(frame, fg_color="transparent")
    main_content.pack(fill="both", expand=True, padx=40, pady=30)

    # Form container with card styling
    form_card = ctk.CTkFrame(main_content, fg_color=CARD_BG, corner_radius=15)
    form_card.pack(fill="both", expand=True)

    form_inner = ctk.CTkFrame(form_card, fg_color="transparent")
    form_inner.pack(fill="both", expand=True, padx=30, pady=30)

    # Title and description
    header_frame = ctk.CTkFrame(form_inner, fg_color="transparent")
    header_frame.pack(fill="x", pady=(0, 20))

    title_label = ctk.CTkLabel(
        header_frame,
        text="Edit Product Details",
        font=("Helvetica", 18, "bold"),
        text_color=SHOPPING,
    )
    title_label.pack(anchor="w")

    subtitle_label = ctk.CTkLabel(
        header_frame,
        text="Update the information below to modify the product",
        font=("Helvetica", 14),
        text_color="gray",
    )
    subtitle_label.pack(anchor="w", pady=(5, 0))

    # Two column layout
    columns_frame = ctk.CTkFrame(form_inner, fg_color="transparent")
    columns_frame.pack(fill="both", expand=True)

    # Left column (form fields)
    left_col = ctk.CTkFrame(columns_frame, fg_color="transparent")
    left_col.pack(side="left", fill="both", expand=True, padx=(0, 15))

    def create_form_field(parent, label_text, placeholder_text):
        field_frame = ctk.CTkFrame(parent, fg_color="transparent")
        field_frame.pack(fill="x", pady=(0, 15))

        label = ctk.CTkLabel(
            field_frame, text=label_text, font=("Helvetica", 14), text_color="white"
        )
        label.pack(anchor="w")

        entry = ctk.CTkEntry(
            field_frame,
            height=40,
            corner_radius=8,
            placeholder_text=placeholder_text,
            fg_color="#3b3b3b",
            border_color=SHOPPING,
            text_color="white",
        )
        entry.pack(fill="x", pady=(5, 0))
        return entry

    # Form Fields
    prod_title_entry = create_form_field(
        left_col, "Product Title", "Enter product title..."
    )
    price_entry = create_form_field(left_col, "Price ($)", "Enter price...")
    stock_entry = create_form_field(
        left_col, "Stock Quantity", "Enter available stock..."
    )

    # Description with multiline entry
    desc_frame = ctk.CTkFrame(left_col, fg_color="transparent")
    desc_frame.pack(fill="x", pady=(0, 15))

    desc_label = ctk.CTkLabel(
        desc_frame, text="Description", font=("Helvetica", 14), text_color="white"
    )
    desc_label.pack(anchor="w")

    desc_entry = ctk.CTkTextbox(
        desc_frame,
        height=100,
        corner_radius=8,
        fg_color="#3b3b3b",
        border_color=SHOPPING,
        text_color="white",
    )
    desc_entry.pack(fill="x", pady=(5, 0))

    # Right column (image upload)
    right_col = ctk.CTkFrame(columns_frame, fg_color="transparent")
    right_col.pack(side="left", fill="both", expand=True, padx=(15, 0))

    image_frame = ctk.CTkFrame(right_col, fg_color="transparent")
    image_frame.pack(fill="x", pady=(0, 15))

    image_label = ctk.CTkLabel(
        image_frame, text="Product Image", font=("Helvetica", 14), text_color="white"
    )
    image_label.pack(anchor="w")

    # Image preview area
    preview_frame = ctk.CTkFrame(
        image_frame, fg_color="#3b3b3b", corner_radius=10, height=200
    )
    preview_frame.pack(fill="x", pady=(5, 0))
    preview_frame.pack_propagate(False)

    preview_label = ctk.CTkLabel(
        preview_frame,
        text="No image selected",
        font=("Helvetica", 14),
        text_color="gray",
    )
    preview_label.pack(expand=True)

    # Track image path and current image URL (to know if we need to upload a new one)
    image_path = ""
    current_image_url = ""

    def browse_image():
        nonlocal image_path
        filename = filedialog.askopenfilename(
            title="Select Product Image",
            filetypes=[("Image Files", "*.png *.jpg *.jpeg *.gif")],
        )
        if filename:
            image_path = filename
            try:
                pil_img = Image.open(filename).resize((150, 150))
                tk_img = ImageTk.PhotoImage(pil_img)
                preview_label.configure(image=tk_img, text="")
                preview_label.image = tk_img

                # Update button text to indicate a new image is selected
                browse_button.configure(text="Change Image (New Image Selected)")
                browse_button.configure(fg_color="#4CAF50")  # Green color
            except Exception as e:
                print(f"Image preview error: {e}")
        else:
            # User canceled selection
            if current_image_url:
                browse_button.configure(text="Change Image (Keeping Current)")
            else:
                browse_button.configure(text="Browse Image (No Image)")

    browse_button = ctk.CTkButton(
        image_frame,
        text="Browse Image",
        command=browse_image,
        corner_radius=8,
        height=40,
        font=("Helvetica", 14),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    browse_button.pack(pady=(10, 0))

    # Buttons at the bottom
    buttons_frame = ctk.CTkFrame(form_inner, fg_color="transparent")
    buttons_frame.pack(fill="x", pady=(20, 0))

    def submit_changes():
        title_val = prod_title_entry.get().strip()
        price_val = price_entry.get().strip()
        desc_val = desc_entry.get("1.0", "end-1c").strip()
        stock_val = stock_entry.get().strip()

        if not title_val or not price_val or not stock_val:
            messagebox.showerror("Error", "Title, Price, and Stock are required")
            return

        # Validate that price and stock are positive numbers
        try:
            price = float(price_val)
            if price <= 0:
                messagebox.showerror("Invalid Price", "Price must be greater than zero")
                price_entry.delete(0, "end")
                price_entry.focus()
                return
        except ValueError:
            messagebox.showerror("Invalid Price", "Price must be a valid number")
            price_entry.delete(0, "end")
            price_entry.focus()
            return

        try:
            stock = int(stock_val)
            if stock <= 0:
                messagebox.showerror(
                    "Invalid Stock", "Stock quantity must be greater than zero"
                )
                stock_entry.delete(0, "end")
                stock_entry.focus()
                return
        except ValueError:
            messagebox.showerror(
                "Invalid Stock", "Stock quantity must be a valid integer"
            )
            stock_entry.delete(0, "end")
            stock_entry.focus()
            return

        # Prepare form data
        data = {
            "name": title_val,
            "price": price_val,
            "description": desc_val,
            "stock": stock_val,
            "shop_id": shop_id,  # Use the loaded shop_id
            "category_id": category_id
            or 1,  # Use the loaded category_id or default to 1
        }

        # Prepare files for upload if a new image was selected
        files = {}
        if image_path and os.path.isfile(image_path):
            # User selected a new image
            files["images"] = open(image_path, "rb")
        else:
            # Backend requires images field even if no new image is selected
            # Use an empty file as a placeholder to satisfy the API requirement
            # The backend will ignore this placeholder and keep existing images
            empty_file = io.BytesIO(b"")
            empty_file.name = (
                "empty.jpg"  # Special filename that backend recognizes as placeholder
            )
            files["images"] = empty_file

        # Make the API request
        headers = {"Authorization": f"Bearer {token}"}
        try:
            # Use PUT for updating existing product
            resp = requests.put(
                f"{API_URL}/product/put/{product_id}",
                data=data,
                files=files,
                headers=headers,
            )
            if resp.status_code == 200:
                messagebox.showinfo("Success", "Product updated successfully!")

                # Refresh both the owner_products and owner_dashboard frames
                # We do this by switching back to owner_products with refresh flag,
                # and setting a timer to switch to owner_dashboard with refresh flag
                root = parent.winfo_toplevel()

                # First, go back to owner_products with refresh flag
                switch_func("owner_products", True)

                # Then, schedule owner_dashboard refresh
                # The user can click to see the dashboard, and it will already be refreshed
                root.after(300, lambda: switch_func("owner_dashboard", True))

                # Finally, switch back to owner_products so user sees the product list
                root.after(400, lambda: switch_func("owner_products"))
            else:
                messagebox.showerror(
                    "Error",
                    f"Failed to update product. Status: {resp.status_code}\n{resp.text}",
                )
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")
        finally:
            if files and "images" in files:
                files["images"].close()

    # Cancel button
    cancel_button = ctk.CTkButton(
        buttons_frame,
        text="Cancel",
        command=go_back,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14),
        fg_color="#555555",
        hover_color="#777777",
    )
    cancel_button.pack(side="left", fill="x", expand=True, padx=(0, 5))

    # Update button
    submit_button = ctk.CTkButton(
        buttons_frame,
        text="Update Product",
        command=submit_changes,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    submit_button.pack(side="right", fill="x", expand=True, padx=(5, 0))

    # Function to load product data
    def load_product_data():
        nonlocal current_image_url, shop_id, category_id, image_path

        if not product_id:
            messagebox.showerror("Error", "No product ID provided")
            go_back()
            return

        headers = {"Authorization": f"Bearer {token}"}
        try:
            resp = requests.get(f"{API_URL}/product/get/{product_id}", headers=headers)
            if resp.status_code == 200:
                product_data = resp.json()

                # Clear form fields first to prevent duplication
                prod_title_entry.delete(0, "end")
                price_entry.delete(0, "end")
                stock_entry.delete(0, "end")
                desc_entry.delete("1.0", "end")

                # Populate form fields with existing data
                prod_title_entry.insert(0, product_data.get("name", ""))
                price_entry.insert(0, str(product_data.get("price", "")))
                stock_entry.insert(0, str(product_data.get("stock", "")))
                desc_entry.insert("1.0", product_data.get("description", ""))

                # Store shop_id and category_id for submission
                shop_id = product_data.get("shop_id")
                category_id = product_data.get("category_id")

                # Reset image path since we're loading a product
                image_path = ""

                # Display existing image if available
                if product_data.get("images") and len(product_data["images"]) > 0:
                    image_url = product_data["images"][0].get("image_url")
                    if image_url:
                        current_image_url = image_url
                        try:
                            resp = requests.get(image_url)
                            if resp.status_code == 200:
                                pil_img = Image.open(io.BytesIO(resp.content)).resize(
                                    (150, 150)
                                )
                                tk_img = ImageTk.PhotoImage(pil_img)
                                preview_label.configure(image=tk_img, text="")
                                preview_label.image = tk_img

                                # Update button text to show existing image
                                browse_button.configure(
                                    text="Change Image (Current Image)"
                                )
                                browse_button.configure(
                                    fg_color=SHOPPING
                                )  # Reset to default color
                        except Exception as e:
                            print(f"Error loading product image: {e}")
                else:
                    # No image available
                    current_image_url = ""
                    preview_label.configure(image=None, text="No image available")
                    browse_button.configure(text="Add Image")
            else:
                messagebox.showerror(
                    "Error",
                    f"Failed to load product data. Status: {resp.status_code}\n{resp.text}",
                )
                go_back()
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")
            go_back()

    # Initialize variables needed for editing
    shop_id = None
    category_id = None

    # Load product data when frame is created
    if product_id:
        load_product_data()

    def refresh_data(new_product_id=None):
        nonlocal product_id
        if new_product_id:
            product_id = new_product_id
            load_product_data()

    frame.refresh_data = refresh_data

    return frame
