import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox
import io
from typing import List, Dict, Any

# UI constants - adding more consistent colors
SHOPPING = "#00c1ff"  # Primary theme color
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
PRIMARY_COLOR = "#0078D7"  # Updated main accent color
SECONDARY_COLOR = "#005A9E"  # Darker accent for hover states
ACCENT_COLOR = "#FF4242"  # Red accent for prices and important actions
HOVER_COLOR = "#FF6B6B"  # Lighter red for hover states
TEXT_COLOR = "#FFFFFF"
BUTTON_CORNER_RADIUS = 8
CARD_CORNER_RADIUS = 10
BACKEND_HOST = "http://127.0.0.1:8000"


def fix_url(url):
    """
    If the provided URL does not start with http, assume it's a relative path.
    Remove any unwanted prefix (e.g., "app/static/") and prepend the public URL.
    """
    print(f"[DEBUG] fix_url received: {url}")
    result = ""

    if not url:
        print("[DEBUG] URL is None or empty")
        return ""

    if url.startswith("http"):
        print(f"[DEBUG] URL already starts with http, returning as is: {url}")
        result = url
    else:
        # If the URL starts with "app/static/", remove that part.
        prefix = "app/static/"
        if url.startswith(prefix):
            url = url[len(prefix) :]
            print(f"[DEBUG] Removed prefix, now: {url}")

        # Prepend the public URL
        result = f"{BACKEND_HOST}/static/{url}"
        print(f"[DEBUG] Final URL after fixing: {result}")

    return result


def view_shop_frame(parent, switch_func, API_URL, token, shop_id):
    """
    CustomTkinter-based frame to display shop details and products.
    """
    frame = ctk.CTkFrame(parent, fg_color="transparent")

    # Store image references at the frame level to prevent garbage collection
    frame.image_references = []

    # Fetch shop details and products from the backend
    headers = {"Authorization": f"Bearer {token}"}
    try:
        # First get shop data
        response = requests.get(f"{API_URL}/shops/get/{shop_id}", headers=headers)
        if response.status_code == 200:
            data = response.json()
            shop_data = data.get("shop", {})
            products_basic = data.get("products", [])

            print(
                f"[DEBUG] Found {len(products_basic)} basic products in shop {shop_id}"
            )

            # Get detailed product data
            products = []
            for product in products_basic:
                product_id = product.get("id")
                try:
                    url = f"{API_URL}/product/get/{product_id}"
                    print(f"[DEBUG] Fetching complete data from {url}")
                    product_resp = requests.get(url, headers=headers)

                    if product_resp.status_code == 200:
                        complete_product = product_resp.json()
                        images = complete_product.get("images", [])
                        print(
                            f"[DEBUG] Product {product_id} data received with {len(images)} images"
                        )

                        # Print each image URL for debugging
                        for i, img in enumerate(images):
                            print(f"[DEBUG] Image {i + 1} URL: {img.get('image_url')}")

                        products.append(complete_product)
                    else:
                        print(
                            f"[DEBUG] Failed to get product {product_id}: {product_resp.status_code}"
                        )
                        products.append(product)  # Fall back to basic product data
                except Exception as e:
                    print(f"[DEBUG] Exception fetching product {product_id}: {str(e)}")
                    products.append(product)  # Fall back to basic product data
        else:
            messagebox.showerror(
                "Error", f"Failed to fetch shop data. Status: {response.status_code}"
            )
            return frame
    except Exception as e:
        messagebox.showerror("Error", f"Request error: {e}")
        return frame

    # ------------- TOP LAYER (Shop Image and Details) -------------
    top_layer = ctk.CTkFrame(frame, fg_color="transparent")
    top_layer.place(relx=0, rely=0, relwidth=1, relheight=0.4)

    # Shop Image
    shop_image_label = ctk.CTkLabel(top_layer, text="")
    shop_image_label.place(relx=0.05, rely=0.1, relwidth=0.3, relheight=0.8)

    shop_image_url = shop_data.get("image_url")
    if shop_image_url:
        try:
            # If URL doesn't start with http, prepend BACKEND_HOST
            if not shop_image_url.startswith("http"):
                if shop_image_url.startswith("app/static/"):
                    shop_image_url = shop_image_url[len("app/static/") :]
                shop_image_url = f"{BACKEND_HOST}/static/{shop_image_url}"

            print(f"[DEBUG] Fetching shop image from: {shop_image_url}")
            resp = requests.get(shop_image_url, timeout=5)

            if resp.status_code == 200:
                pil_img = Image.open(io.BytesIO(resp.content)).resize((200, 200))
                tk_img = ImageTk.PhotoImage(pil_img)
                shop_image_label.configure(image=tk_img, text="")

                # Store the image reference at the frame level
                if not hasattr(frame, "image_references"):
                    frame.image_references = []
                frame.image_references.append(tk_img)

                print(f"[DEBUG] Successfully set shop image")
            else:
                print(f"[DEBUG] Failed to load shop image: {resp.status_code}")
        except Exception as e:
            print(f"[DEBUG] Failed to load shop image: {e}")

    # Shop Details
    shop_details_frame = ctk.CTkFrame(top_layer, fg_color="transparent")
    shop_details_frame.place(relx=0.4, rely=0.1, relwidth=0.55, relheight=0.8)

    shop_name_label = ctk.CTkLabel(
        shop_details_frame,
        text=shop_data.get("name", "Shop Name"),
        font=("Helvetica", 20, "bold"),
    )
    shop_name_label.pack(pady=10)

    shop_description_label = ctk.CTkLabel(
        shop_details_frame,
        text=shop_data.get("description", "No description available."),
        font=("Helvetica", 14),
        wraplength=400,
        justify="left",
    )
    shop_description_label.pack(pady=5)

    shop_address_label = ctk.CTkLabel(
        shop_details_frame,
        text=f"Address: {shop_data.get('address', 'No address provided')}",
        font=("Helvetica", 12),
    )
    shop_address_label.pack(pady=5)

    # ------------- BOTTOM LAYER (Products) -------------
    bottom_layer = ctk.CTkFrame(frame, fg_color="transparent")
    bottom_layer.place(relx=0, rely=0.4, relwidth=1, relheight=0.6)

    products_label = ctk.CTkLabel(
        bottom_layer, text="Products", font=("Helvetica", 16, "bold")
    )
    products_label.pack(pady=10)

    products_frame = ctk.CTkScrollableFrame(
        bottom_layer, fg_color="transparent", width=750, height=300
    )
    products_frame.pack(fill="both", expand=True, padx=10, pady=5)

    def create_product_card(parent, product_data):
        card_width = 130
        card_height = 210
        card = ctk.CTkFrame(
            parent,
            corner_radius=CARD_CORNER_RADIUS,
            fg_color=CARD_BG,
            width=card_width,
            height=card_height,
        )
        card.pack_propagate(False)

        # Create a frame specifically for the image
        image_frame = ctk.CTkFrame(card, fg_color="transparent", width=100, height=80)
        image_frame.pack(pady=5)
        image_frame.pack_propagate(False)  # Prevent size changes

        # Create the image label inside the frame
        image_label = ctk.CTkLabel(image_frame, text="")
        image_label.place(relx=0.5, rely=0.5, anchor="center")  # Center in frame

        product_images = product_data.get("images", [])
        product_id = product_data.get("id", "unknown")
        print(
            f"[DEBUG] Creating card for product {product_id}, found {len(product_images)} images"
        )

        # Store the image reference on the card itself to prevent garbage collection
        card.tk_img = None

        if product_images:
            img_url = product_images[0].get("image_url")
            print(f"[DEBUG] Product {product_id} image URL: {img_url}")
            if img_url:
                try:
                    # If URL doesn't start with http, prepend BACKEND_HOST
                    if not img_url.startswith("http"):
                        if img_url.startswith("app/static/"):
                            img_url = img_url[len("app/static/") :]
                        img_url = f"{BACKEND_HOST}/static/{img_url}"

                    print(f"[DEBUG] Fetching image from: {img_url}")
                    resp = requests.get(img_url, timeout=5)  # Add timeout
                    print(f"[DEBUG] Image fetch status: {resp.status_code}")

                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize((70, 70))
                        tk_img = ImageTk.PhotoImage(pil_img)

                        # Store the reference at card level to prevent garbage collection
                        card.tk_img = tk_img

                        image_label.configure(image=tk_img, text="")
                        print(
                            f"[DEBUG] Successfully set image for product {product_id}"
                        )
                    else:
                        print(f"[DEBUG] Failed to fetch image: {resp.text[:100]}")
                except Exception as e:
                    print(f"[DEBUG] Product image error: {e}")
        else:
            print(f"[DEBUG] No images found for product {product_id}")

        name_text = product_data.get("name", "No Name")
        name_label = ctk.CTkLabel(
            card,
            text=name_text,
            font=("Helvetica", 11, "bold"),
            wraplength=120,
            anchor="center",
            justify="center",
        )
        name_label.pack(padx=5, pady=3)

        price_val = product_data.get("price", 0.0)
        price_label = ctk.CTkLabel(
            card,
            text=f"${price_val:,.2f}",
            font=("Helvetica", 10, "bold"),
            text_color=ACCENT_COLOR,
        )
        price_label.pack(side="bottom", pady=5)

        # Add view details button instead of click handler
        view_button = ctk.CTkButton(
            card,
            text="View",
            width=80,
            height=25,
            corner_radius=BUTTON_CORNER_RADIUS,
            font=("Helvetica", 10, "bold"),
            fg_color=SHOPPING,
            text_color="white",
            hover_color="#008dd1",
            command=lambda: view_product_details(product_data),
        )
        view_button.pack(side="bottom", pady=5)

        return card

    def view_product_details(product_data):
        # Fetch complete product before viewing it
        headers = {"Authorization": f"Bearer {token}"}
        try:
            product_id = product_data.get("id")
            print(f"[DEBUG] Fetching complete details for product ID: {product_id}")

            # Make sure we get the complete product data with images
            resp = requests.get(f"{API_URL}/product/get/{product_id}", headers=headers)

            if resp.status_code == 200:
                complete_product = resp.json()

                # Debug log to verify images are in the response
                images = complete_product.get("images", [])
                print(f"[DEBUG] Received product data with {len(images)} images")
                if images:
                    print(f"[DEBUG] First image URL: {images[0].get('image_url')}")

                # Use the complete data for view_product
                print(f"[DEBUG] Switching to view_product with complete data")
                switch_func("view_product", complete_product)
            else:
                print(
                    f"[DEBUG] Failed to fetch product {product_id}: {resp.status_code}"
                )
                messagebox.showerror(
                    "Error",
                    f"Failed to fetch product details. Status: {resp.status_code}",
                )
                # Still try to show the product with available data as fallback
                switch_func("view_product", product_data)
        except Exception as e:
            print(f"[DEBUG] Exception in view_product_details: {str(e)}")
            messagebox.showerror("Error", f"Request error: {e}")
            # Fallback to original data if exception occurs
            switch_func("view_product", product_data)

    def display_products(products, container):
        for widget in container.winfo_children():
            widget.destroy()

        if not products:
            ctk.CTkLabel(container, text="No products found.").pack(pady=10)
            return

        print(f"[DEBUG] Displaying {len(products)} products in shop view")

        grid_frame = ctk.CTkFrame(container, fg_color="transparent")
        grid_frame.pack(fill="both", expand=True)

        # We'll store image references here to prevent garbage collection
        frame.image_references = []

        # Get complete product data with images for each product
        complete_products = []
        for product in products:
            product_id = product.get("id")
            # Check if this product already has images
            if product.get("images"):
                print(f"[DEBUG] Product {product_id} already has images")
                complete_products.append(product)
            else:
                # Fetch complete product data with images
                try:
                    print(f"[DEBUG] Fetching complete data for product {product_id}")
                    product_resp = requests.get(
                        f"{API_URL}/product/get/{product_id}", headers=headers
                    )
                    if product_resp.status_code == 200:
                        complete_product = product_resp.json()
                        images = complete_product.get("images", [])
                        print(
                            f"[DEBUG] Received product {product_id} with {len(images)} images"
                        )
                        complete_products.append(complete_product)
                    else:
                        print(
                            f"[DEBUG] Failed to get complete product {product_id}: {product_resp.status_code}"
                        )
                        complete_products.append(product)  # Use original as fallback
                except Exception as e:
                    print(f"[DEBUG] Error fetching product {product_id}: {e}")
                    complete_products.append(product)  # Use original as fallback

        # Now display the products with their complete data
        print(f"[DEBUG] Displaying {len(complete_products)} complete products")
        columns = 5  # 5 products per row
        for idx, product in enumerate(complete_products):
            row = idx // columns
            col = idx % columns
            print(
                f"[DEBUG] Creating card for product {product.get('id')} with {len(product.get('images', []))} images"
            )
            product_card = create_product_card(grid_frame, product)

            # If the card has an image reference, store it at the frame level too
            if hasattr(product_card, "tk_img") and product_card.tk_img:
                frame.image_references.append(product_card.tk_img)

            product_card.grid(row=row, column=col, padx=5, pady=5)

    # Display products
    display_products(products, products_frame)

    return frame


def view_product_frame(
    parent, switch_func, API_URL, token, product_data: Dict[Any, Any] = None
):
    """
    CustomTkinter-based frame to display detailed product information
    and provide an "Add to Cart" function.

    Parameters:
    - parent: Parent widget
    - switch_func: Function to switch between frames
    - API_URL: API base URL
    - token: User authentication token
    - product_data: Product data dictionary from the API
    """
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token

    # --- Top navigation bar ---
    top_bar = ctk.CTkFrame(frame, fg_color=PRIMARY_COLOR, height=50)
    top_bar.pack(fill="x")

    nav_label = ctk.CTkLabel(
        top_bar,
        text="Product Details",
        font=("Helvetica", 18, "bold"),
        text_color=TEXT_COLOR,
    )
    nav_label.pack(side="left", padx=20, pady=10)

    def navigate_back():
        if product_data and "shop_id" in product_data:
            switch_func("view_shop", product_data["shop_id"])
        else:
            switch_func("shop")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        width=80,
        font=("Helvetica", 14),
        fg_color="transparent",
        text_color=TEXT_COLOR,
        hover_color=SECONDARY_COLOR,
        command=navigate_back,
    )
    back_button.pack(side="right", padx=20, pady=10)

    # --- Main content area ---
    main_content = ctk.CTkFrame(frame, fg_color=DARK_BG)
    main_content.pack(fill="both", expand=True, padx=20, pady=20)

    # Product data variables (will be populated by refresh_data)
    product_id = None
    product_name = ""
    product_price = 0.0
    product_stock = 0
    product_desc = ""
    product_images = []
    quantity = 1  # Default quantity for cart

    def refresh_data(new_product_data: Dict[Any, Any] = None):
        """Update product data and refresh display"""
        nonlocal \
            product_data, \
            product_id, \
            product_name, \
            product_price, \
            product_stock, \
            product_desc, \
            product_images

        if new_product_data is not None:
            product_data = new_product_data

        if product_data:
            product_id = product_data.get("id")
            product_name = product_data.get("name", "")
            product_price = product_data.get("price", 0.0)
            product_stock = product_data.get("stock", 0)
            product_desc = product_data.get("description", "")
            product_images = product_data.get("images", [])

            # Check if images are missing and fetch complete data if needed
            if not product_images and product_id:
                print(
                    f"[DEBUG] No images found in product data, fetching complete data for ID: {product_id}"
                )
                try:
                    headers = {"Authorization": f"Bearer {token}"}
                    resp = requests.get(
                        f"{API_URL}/product/get/{product_id}", headers=headers
                    )
                    if resp.status_code == 200:
                        complete_data = resp.json()
                        complete_images = complete_data.get("images", [])
                        print(
                            f"[DEBUG] Fetched complete product data with {len(complete_images)} images"
                        )

                        # Update our product data with the complete data
                        product_data = complete_data
                        product_images = complete_images

                        # Also update other fields just to be safe
                        product_name = complete_data.get("name", product_name)
                        product_price = complete_data.get("price", product_price)
                        product_stock = complete_data.get("stock", product_stock)
                        product_desc = complete_data.get("description", product_desc)
                except Exception as e:
                    print(f"[DEBUG] Error fetching complete product data: {e}")

            # Now update the UI
            update_product_display()

    def update_product_display():
        """Update all UI elements with current product data"""
        # Clear existing widgets
        for widget in main_content.winfo_children():
            widget.destroy()

        print(f"[DEBUG] Updating product display for ID: {product_id}")
        print(f"[DEBUG] Product has {len(product_images)} images")

        # Store image references at the frame level to prevent garbage collection
        if not hasattr(frame, "image_references"):
            frame.image_references = []

        # Create layout structure: image on left, details on right
        image_frame = ctk.CTkFrame(main_content, fg_color=DARK_BG)
        image_frame.pack(side="left", fill="both", expand=True, padx=(0, 10))

        details_frame = ctk.CTkFrame(main_content, fg_color=DARK_BG)
        details_frame.pack(side="right", fill="both", expand=True, padx=(10, 0))

        # --- Product Image Section ---
        image_container = ctk.CTkFrame(
            image_frame, fg_color=CARD_BG, corner_radius=CARD_CORNER_RADIUS
        )
        image_container.pack(fill="both", expand=True, pady=5)

        image_label = ctk.CTkLabel(image_container, text="No image available")
        image_label.pack(fill="both", expand=True, padx=10, pady=10)

        # Try to load the product image
        if product_images and len(product_images) > 0:
            image_url = product_images[0].get("image_url")
            print(f"[DEBUG] Product image URL: {image_url}")

            if image_url:
                try:
                    # If URL doesn't start with http, prepend BACKEND_HOST
                    if not image_url.startswith("http"):
                        if image_url.startswith("app/static/"):
                            image_url = image_url[len("app/static/") :]
                        image_url = f"{BACKEND_HOST}/static/{image_url}"

                    print(f"[DEBUG] Fetching product image from: {image_url}")
                    resp = requests.get(image_url, timeout=5)
                    print(f"[DEBUG] Image response status: {resp.status_code}")

                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize(
                            (300, 300)
                        )
                        tk_img = ImageTk.PhotoImage(pil_img)

                        # Store the image reference to prevent garbage collection
                        frame.image_references.append(tk_img)

                        image_label.configure(image=tk_img, text="")
                        print(f"[DEBUG] Successfully set product image")
                    else:
                        print(
                            f"[DEBUG] Failed to fetch image: {resp.text[:100] if resp.text else None}"
                        )
                except Exception as e:
                    print(f"[DEBUG] Error loading product image: {e}")
        else:
            print(f"[DEBUG] No images found for product {product_id}")

        # --- Product Details Section ---
        details_container = ctk.CTkFrame(
            details_frame, fg_color=CARD_BG, corner_radius=CARD_CORNER_RADIUS
        )
        details_container.pack(fill="both", expand=True, pady=5)

        details_inner = ctk.CTkFrame(details_container, fg_color="transparent")
        details_inner.pack(fill="both", expand=True, padx=20, pady=20)

        # Product name
        name_label = ctk.CTkLabel(
            details_inner,
            text=product_name,
            font=("Helvetica", 22, "bold"),
            text_color=TEXT_COLOR,
            anchor="w",
        )
        name_label.pack(fill="x", pady=(0, 15))

        # Price
        price_label = ctk.CTkLabel(
            details_inner,
            text=f"${product_price:.2f}",
            font=("Helvetica", 18, "bold"),
            text_color=ACCENT_COLOR,
            anchor="w",
        )
        price_label.pack(fill="x", pady=(0, 20))

        # Stock information
        stock_status = "In Stock" if product_stock > 0 else "Out of Stock"
        stock_color = "#4CAF50" if product_stock > 0 else "#F44336"  # Green or Red

        stock_frame = ctk.CTkFrame(details_inner, fg_color="transparent", height=30)
        stock_frame.pack(fill="x", pady=(0, 15))

        # Modified stock label to show remaining quantity when in stock
        if product_stock > 0:
            stock_text = f"Status: {stock_status} ({product_stock} remaining)"
        else:
            stock_text = f"Status: {stock_status}"

        stock_label = ctk.CTkLabel(
            stock_frame,
            text=stock_text,
            font=("Helvetica", 14),
            text_color=stock_color,
        )
        stock_label.pack(side="left")

        if product_stock > 0:
            quantity_label = ctk.CTkLabel(
                stock_frame,
                text=f"Quantity:",
                font=("Helvetica", 14),
                text_color=TEXT_COLOR,
            )
            quantity_label.pack(side="left", padx=(20, 10))

            # Quantity selector
            qty_selector_frame = ctk.CTkFrame(stock_frame, fg_color="transparent")
            qty_selector_frame.pack(side="left")

            nonlocal quantity
            quantity = 1  # Reset to 1 when display is updated

            def update_quantity(delta):
                nonlocal quantity
                new_qty = max(1, min(product_stock, quantity + delta))
                if new_qty != quantity:
                    quantity = new_qty
                    qty_display.configure(text=str(quantity))

            minus_btn = ctk.CTkButton(
                qty_selector_frame,
                text="-",
                width=25,
                height=25,
                font=("Helvetica", 14, "bold"),
                fg_color="#555555",
                hover_color="#777777",
                command=lambda: update_quantity(-1),
            )
            minus_btn.pack(side="left")

            qty_display = ctk.CTkLabel(
                qty_selector_frame,
                text=str(quantity),
                width=30,
                font=("Helvetica", 14),
                text_color=TEXT_COLOR,
            )
            qty_display.pack(side="left", padx=5)

            plus_btn = ctk.CTkButton(
                qty_selector_frame,
                text="+",
                width=25,
                height=25,
                font=("Helvetica", 14, "bold"),
                fg_color="#555555",
                hover_color="#777777",
                command=lambda: update_quantity(1),
            )
            plus_btn.pack(side="left")

            # "Add to Cart" button
            def add_to_cart():
                if not token:
                    if messagebox.askyesno(
                        "Login Required",
                        "You need to be logged in to add items to your cart. Do you want to log in now?",
                    ):
                        switch_func("login")
                    return

                # Validate quantity again just to be safe
                if quantity < 1 or quantity > product_stock:
                    messagebox.showerror(
                        "Invalid Quantity", "Please select a valid quantity"
                    )
                    return

                # Prepare data for API call
                headers = {"Authorization": f"Bearer {token}"}
                data = {
                    "product_id": product_id,
                    "shop_id": product_data.get("shop_id"),
                    "quantity": quantity,
                }

                try:
                    resp = requests.post(
                        f"{API_URL}/cart/add", json=data, headers=headers
                    )
                    if resp.status_code == 200:
                        messagebox.showinfo(
                            "Success", "Item added to cart successfully!"
                        )
                        # Navigate to the My Orders page
                        switch_func("user_orders")
                    else:
                        messagebox.showerror(
                            "Error",
                            f"Failed to add item to cart. Status: {resp.status_code}\n{resp.text}",
                        )
                except Exception as e:
                    messagebox.showerror("Error", f"Request error: {e}")

            add_to_cart_btn = ctk.CTkButton(
                details_inner,
                text="Add to Cart",
                height=45,
                corner_radius=BUTTON_CORNER_RADIUS,
                font=("Helvetica", 16, "bold"),
                fg_color=ACCENT_COLOR,
                hover_color=HOVER_COLOR,
                command=add_to_cart,
            )
            add_to_cart_btn.pack(fill="x", pady=(5, 20))

        # Description section
        desc_label = ctk.CTkLabel(
            details_inner,
            text="Description",
            font=("Helvetica", 16, "bold"),
            text_color=TEXT_COLOR,
            anchor="w",
        )
        desc_label.pack(fill="x", pady=(10, 5))

        desc_area = ctk.CTkTextbox(
            details_inner,
            fg_color="#252525",
            corner_radius=5,
            height=120,
        )
        desc_area.pack(fill="both", expand=True)
        desc_area.insert("1.0", product_desc or "No description available.")
        desc_area.configure(state="disabled")  # Make read-only

    # Initial display update if we have product data
    if product_data:
        refresh_data()

    frame.refresh_data = refresh_data
    return frame
