import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox
import io

# UI Constants
SHOPPING = "#00c1ff"
BACKEND_HOST = "http://127.0.0.1:8000"


def fix_url(url):
    """
    Fix the URL to ensure it points to the correct static file location.
    """
    if url.startswith("http"):
        return url
    # If the URL starts with "app/static/", remove that part
    prefix = "app/static/"
    if url.startswith(prefix):
        url = url[len(prefix) :]
    return f"{BACKEND_HOST}/static/{url}"


def view_shop_frame(parent, switch_func, API_URL, token, shop_id):
    """
    CustomTkinter-based frame to display shop details and products.
    """
    # Main frame
    main_frame = ctk.CTkFrame(parent, fg_color="transparent")
    main_frame.pack(fill="both", expand=True)

    # Track if user is a shop owner
    is_owner = False

    # Check if user is an owner
    def check_user_role():
        nonlocal is_owner
        if not token:
            return

        headers = {"Authorization": f"Bearer {token}"}
        try:
            resp = requests.get(f"{API_URL}/user/profile", headers=headers)
            if resp.status_code == 200:
                data = resp.json()
                user_role = data.get("role", "")
                is_owner = user_role == "owner" or user_role == "shop_owner"
        except Exception as e:
            print(f"Error checking user role: {e}")

    # Create a container frame for content that can be refreshed
    container_frame = ctk.CTkFrame(main_frame, fg_color="transparent")
    container_frame.pack(fill="both", expand=True)

    def refresh_data(new_shop_id=None, new_token=None):
        """Refresh the shop view with new data"""
        nonlocal token, shop_id
        if new_token:
            token = new_token
        if new_shop_id:
            shop_id = new_shop_id

        # Check user role with the updated token
        check_user_role()

        # Clear the container
        for widget in container_frame.winfo_children():
            widget.destroy()

        if not shop_id:
            print("[DEBUG] No shop ID provided")
            no_shop_label = ctk.CTkLabel(
                container_frame, text="No shop selected", font=("Helvetica", 16)
            )
            no_shop_label.pack(pady=20)
            return

        # Back button at the top with dashboard_bypass for owners
        def go_to_dashboard():
            # For shop owners, use dashboard_bypass to ensure they go to the user dashboard
            if is_owner:
                switch_func("dashboard_bypass", token)
            else:
                switch_func("dashboard")

        back_button = ctk.CTkButton(
            container_frame,
            text="← Back to Dashboard",
            command=go_to_dashboard,
            fg_color=SHOPPING,
            text_color="white",
            width=150,
        )
        back_button.pack(anchor="nw", padx=20, pady=10)

        # Create a scrollable frame for all content
        content_frame = ctk.CTkScrollableFrame(container_frame, fg_color="transparent")
        content_frame.pack(fill="both", expand=True, padx=20, pady=10)

        # Fetch shop details and products from the backend
        headers = {"Authorization": f"Bearer {token}"}
        try:
            print(f"[DEBUG] Fetching shop data for ID: {shop_id}")
            response = requests.get(f"{API_URL}/shops/get/{shop_id}", headers=headers)
            print(f"[DEBUG] Response status: {response.status_code}")

            if response.status_code == 200:
                data = response.json()
                shop_data = data.get("shop", {})
                products_basic = data.get("products", [])
                print(f"[DEBUG] Shop data: {shop_data}")
                print(f"[DEBUG] Number of basic products: {len(products_basic)}")

                # Fetch complete product data with images
                products = []
                for product in products_basic:
                    product_id = product.get("id")
                    try:
                        print(
                            f"[DEBUG] Fetching complete data for product {product_id}"
                        )
                        product_resp = requests.get(
                            f"{API_URL}/product/get/{product_id}", headers=headers
                        )
                        if product_resp.status_code == 200:
                            complete_product = product_resp.json()
                            images = complete_product.get("images", [])
                            print(
                                f"[DEBUG] Received product {product_id} with {len(images)} images"
                            )
                            products.append(complete_product)
                        else:
                            print(
                                f"[DEBUG] Failed to get complete product {product_id}: {product_resp.status_code}"
                            )
                            products.append(product)  # Use original as fallback
                    except Exception as e:
                        print(f"[DEBUG] Error fetching product {product_id}: {e}")
                        products.append(product)  # Use original as fallback

                print(
                    f"[DEBUG] Final number of products with complete data: {len(products)}"
                )
            else:
                print(f"[DEBUG] Error response: {response.text}")
                messagebox.showerror(
                    "Error",
                    f"Failed to fetch shop data. Status: {response.status_code}",
                )
                return
        except Exception as e:
            print(f"[DEBUG] Error fetching shop data: {e}")
            messagebox.showerror("Error", f"Request error: {e}")
            return

        # Shop Header Section
        shop_header = ctk.CTkFrame(content_frame, fg_color="#2b2b2b", corner_radius=10)
        shop_header.pack(fill="x", padx=10, pady=10)

        # Main frame to store references
        main_frame.image_references = []

        # Shop Image and Details in header
        header_left = ctk.CTkFrame(shop_header, fg_color="transparent")
        header_left.pack(side="left", padx=20, pady=20)

        shop_image_label = ctk.CTkLabel(header_left, text="")
        shop_image_label.pack(pady=5)

        # Load shop image
        shop_image_url = shop_data.get("image_url")
        if shop_image_url:
            try:
                # If URL doesn't start with http, prepend BACKEND_HOST
                if not shop_image_url.startswith("http"):
                    if shop_image_url.startswith("app/static/"):
                        shop_image_url = shop_image_url[len("app/static/") :]
                    shop_image_url = f"{BACKEND_HOST}/static/{shop_image_url}"

                print(f"[DEBUG] Loading shop image from: {shop_image_url}")
                resp = requests.get(shop_image_url, timeout=5)
                if resp.status_code == 200:
                    pil_img = Image.open(io.BytesIO(resp.content)).resize((150, 150))
                    tk_img = ImageTk.PhotoImage(pil_img)
                    shop_image_label.configure(image=tk_img, text="")
                    # Store reference to prevent garbage collection
                    main_frame.image_references.append(tk_img)
                    print(f"[DEBUG] Successfully set shop image")
                else:
                    print(
                        f"[DEBUG] Failed to load shop image. Status: {resp.status_code}"
                    )
            except Exception as e:
                print(f"[DEBUG] Shop image error: {e}")

        # Shop Details
        header_right = ctk.CTkFrame(shop_header, fg_color="transparent")
        header_right.pack(side="left", fill="both", expand=True, padx=20, pady=20)

        shop_name = ctk.CTkLabel(
            header_right,
            text=shop_data.get("name", "Shop Name"),
            font=("Helvetica", 24, "bold"),
            text_color=SHOPPING,
        )
        shop_name.pack(anchor="w")

        shop_description = ctk.CTkLabel(
            header_right,
            text=shop_data.get("description", "No description available."),
            font=("Helvetica", 14),
            wraplength=500,
            justify="left",
        )
        shop_description.pack(anchor="w", pady=10)

        shop_address = ctk.CTkLabel(
            header_right,
            text=f"📍 {shop_data.get('address', 'No address provided')}",
            font=("Helvetica", 12),
            justify="left",
        )
        shop_address.pack(anchor="w")

        # Products Section
        products_section = ctk.CTkFrame(content_frame, fg_color="transparent")
        products_section.pack(fill="both", expand=True, padx=10, pady=10)

        # Products Header
        products_header = ctk.CTkFrame(products_section, fg_color="transparent")
        products_header.pack(fill="x", pady=10)

        products_title = ctk.CTkLabel(
            products_header,
            text="Products",
            font=("Helvetica", 20, "bold"),
            text_color=SHOPPING,
        )
        products_title.pack(side="left")

        products_count = ctk.CTkLabel(
            products_header, text=f"({len(products)} items)", font=("Helvetica", 14)
        )
        products_count.pack(side="left", padx=10)

        # Products Grid
        products_grid = ctk.CTkFrame(products_section, fg_color="transparent")
        products_grid.pack(fill="both", expand=True)

        def create_product_card(parent, product_data):
            card = ctk.CTkFrame(
                parent, fg_color="#2b2b2b", corner_radius=10, width=200, height=280
            )
            card.pack_propagate(False)

            # Store image reference directly on the card to prevent garbage collection
            card.tk_img = None

            # Product Image
            image_frame = ctk.CTkFrame(card, fg_color="transparent", height=150)
            image_frame.pack(fill="x", padx=10, pady=10)
            image_frame.pack_propagate(False)

            image_label = ctk.CTkLabel(image_frame, text="")
            image_label.pack(expand=True)

            # Load product image
            product_id = product_data.get("id", "unknown")
            product_images = product_data.get("images", [])
            print(
                f"[DEBUG] Creating card for product {product_id}, found {len(product_images)} images"
            )

            if product_images:
                img_url = product_images[0].get("image_url")
                print(f"[DEBUG] Product {product_id} image URL: {img_url}")
                if img_url:
                    try:
                        # If URL doesn't start with http, prepend BACKEND_HOST
                        if not img_url.startswith("http"):
                            if img_url.startswith("app/static/"):
                                img_url = img_url[len("app/static/") :]
                            img_url = f"{BACKEND_HOST}/static/{img_url}"

                        print(f"[DEBUG] Fetching image from: {img_url}")
                        resp = requests.get(img_url, timeout=5)
                        print(f"[DEBUG] Image fetch status: {resp.status_code}")

                        if resp.status_code == 200:
                            pil_img = Image.open(io.BytesIO(resp.content)).resize(
                                (120, 120)
                            )
                            tk_img = ImageTk.PhotoImage(pil_img)

                            # Store the reference directly on the card
                            card.tk_img = tk_img

                            image_label.configure(image=tk_img, text="")
                            print(
                                f"[DEBUG] Successfully set image for product {product_id}"
                            )
                        else:
                            print(
                                f"[DEBUG] Failed to fetch image: {resp.text[:100] if resp.text else None}"
                            )
                    except Exception as e:
                        print(f"[DEBUG] Product image error: {e}")
            else:
                print(f"[DEBUG] No images found for product {product_id}")

            # Product Details
            details_frame = ctk.CTkFrame(card, fg_color="transparent")
            details_frame.pack(fill="both", expand=True, padx=10, pady=(0, 10))

            name_label = ctk.CTkLabel(
                details_frame,
                text=product_data.get("name", "No Name"),
                font=("Helvetica", 14, "bold"),
                wraplength=180,
                justify="left",
            )
            name_label.pack(anchor="w")

            price_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
            price_frame.pack(fill="both", expand=True, padx=10, pady=(0, 10))

            price_label = ctk.CTkLabel(
                price_frame,
                text=f"${product_data.get('price', 0.0):.2f}",
                font=("Helvetica", 14, "bold"),
                text_color=SHOPPING,
            )
            price_label.pack(anchor="w", pady=5)

            # View Product Button
            view_button = ctk.CTkButton(
                details_frame,
                text="View Details",
                command=lambda p=product_data: switch_func("view_product", p),
                fg_color=SHOPPING,
                text_color="white",
                height=30,
            )
            view_button.pack(side="bottom", pady=5)

            return card

        if not products:
            no_products_label = ctk.CTkLabel(
                products_grid,
                text="No products available in this shop.",
                font=("Helvetica", 14),
                text_color="gray",
            )
            no_products_label.pack(pady=20)
        else:
            row = 0
            col = 0
            max_cols = 4  # Number of products per row

            for product in products:
                product_card = create_product_card(products_grid, product)

                # Store image reference at frame level if card has one
                if hasattr(product_card, "tk_img") and product_card.tk_img:
                    main_frame.image_references.append(product_card.tk_img)

                product_card.grid(row=row, column=col, padx=10, pady=10, sticky="nsew")

                col += 1
                if col >= max_cols:
                    col = 0
                    row += 1

            # Configure grid columns to be equal width
            for i in range(max_cols):
                products_grid.grid_columnconfigure(i, weight=1)

    # Add the refresh_data method to the frame
    main_frame.refresh_data = refresh_data

    # Initial load with empty shop_id
    refresh_data()

    return main_frame
