import sys
import os

# Add the current directory to the path
sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import customtkinter as ctk
from components.auth.login import login_frame
from components.auth.register import register_frame
from components.shop.create_shop import create_shop_frame
from components.shop.view_shop import view_shop_frame
from components.product.create_product import create_product_frame
from components.product.view_product import view_product_frame
from components.product.edit_product import edit_product_frame
from components.admin.category import category_frame
from components.dashboard import dashboard_frame
from components.user_details import user_details_frame
from components.user_orders import user_orders_frame
from components.user_payments import user_payments_frame
from components.owner.owner_dashboard import owner_dashboard_frame
from components.owner.owner_products import owner_products_frame
from components.owner.owner_orders import owner_orders_frame
import traceback
import sys

API_URL = "http://127.0.0.1:8000"
access_token = None  # Global token
user_role = None  # Global user role


def logout():
    """Clear the global token and redirect to login page"""
    global access_token, user_role
    access_token = None
    user_role = None
    switch_frame("login")


def switch_frame(frame_name, *args):
    """
    Switch between frames.
    Optionally, additional data (like shop_data) can be passed as extra arguments.
    """
    global access_token, user_role

    print(f"switch_frame called with frame_name={frame_name}, args={args}")

    # Special case for bypassing role checks (when directly accessing the dashboard)
    if frame_name == "dashboard_bypass":
        print("*** SHOP OWNER EXPLICITLY ACCESSING USER DASHBOARD ***")
        # Get the token from args if available
        if args and isinstance(args[0], str):
            token_to_use = args[0]
        else:
            token_to_use = access_token

        print(
            f"Rebuilding dashboard frame with token: {token_to_use[:10] if token_to_use else 'None'}"
        )

        # Create a fresh dashboard frame to avoid any stale state
        frames["dashboard"] = dashboard_frame(root, switch_frame, API_URL, token_to_use)
        frames["dashboard"].place(relx=0, rely=0, relwidth=1, relheight=1)

        # Show the dashboard without going through normal routing
        frames["dashboard"].tkraise()
        print("User dashboard displayed successfully")
        return  # Exit the function to avoid any role-based routing

    # Set default for regular flow
    skip_role_check = False

    # Regular flow continues below
    # Special handling for dashboard after login
    if frame_name == "dashboard" and args and isinstance(args[0], str):
        # This is coming from login with a new token
        access_token = args[0]
        print(f"New login detected. Setting token: {access_token[:10]}...")

        # Check user role
        import requests

        headers = {"Authorization": f"Bearer {access_token}"}
        try:
            print(f"Fetching user role from API: {API_URL}/auth/role")
            resp = requests.get(f"{API_URL}/auth/role", headers=headers)
            print(f"Role check response: {resp.status_code}")

            if resp.status_code == 200:
                role_data = resp.json()
                user_role = role_data.get("role")
                print(f"User role detected: {user_role}")

                # Redirect to owner dashboard if user is a shop owner
                if user_role == "shop_owner":
                    print("User is a shop owner, redirecting to owner_dashboard")
                    frame_name = "owner_dashboard"
                else:
                    print(
                        f"User is not a shop owner (role: {user_role}), proceeding to dashboard"
                    )
            else:
                print(f"Failed to get user role. Response: {resp.text}")
        except Exception as e:
            print(f"Error checking user role: {e}")

        # Recreate all frames that need the token to ensure they start with the correct token
        print("Initializing authenticated frames with new token")
        initialize_authenticated_frames(access_token)
    # For other frames, update token if provided
    elif (
        args
        and args[0]
        and isinstance(args[0], str)
        and frame_name not in ["login", "register"]
    ):
        access_token = args[0]
        print(f"Token updated in switch_frame: {access_token[:10]}...")
        # Update token for all frames that need it
        for frame_key, frame_obj in frames.items():
            if hasattr(frame_obj, "update_token"):
                print(f"Updating token for frame: {frame_key}")
                frame_obj.update_token(access_token)

    # Role-based routing
    print(
        f"Checking role-based routing. Current user_role: {user_role}, target frame: {frame_name}"
    )

    # Check if we're coming from an owner page trying to access the dashboard
    owner_going_to_shop = False
    if len(args) > 1 and args[1] == "from_owner_page" and frame_name == "dashboard":
        owner_going_to_shop = True
        print("Owner explicitly navigating to shop dashboard")

    # Only apply role-based routing if not bypassing
    if not skip_role_check:
        if (
            user_role == "shop_owner"
            and frame_name == "dashboard"
            and not owner_going_to_shop
        ):
            # Redirect shop owners to the owner dashboard, but only if not explicitly navigating to shop
            print("Redirecting shop owner from dashboard to owner_dashboard")
            frame_name = "owner_dashboard"
        elif user_role != "shop_owner" and frame_name == "owner_dashboard":
            # Prevent non-owners from accessing owner dashboard
            print(
                f"Preventing non-owner (role: {user_role}) from accessing owner_dashboard"
            )
            frame_name = "dashboard"

    frame = frames.get(frame_name)
    if frame is None:
        print(f"Frame {frame_name} not found!")
        return

    # If there's a refresh method on the frame and we have arguments, call it
    if hasattr(frame, "refresh_data") and len(args) > 0:
        print(f"Calling refresh_data on {frame_name}")
        try:
            frame.refresh_data(*args)
        except Exception as e:
            print(f"Error calling refresh_data on {frame_name}: {e}")
            # Continue despite the error

    # Make sure we have a valid token for authenticated pages
    if frame_name not in ["login", "register"] and (
        access_token is None or access_token == ""
    ):
        print("No valid token, redirecting to login")
        frames["login"].tkraise()
        return

    # Clean up any active auto-refresh for owner dashboard when switching to a different frame
    current_frame = None
    for f in frames.values():
        if f.winfo_viewable():
            current_frame = f
            break

    if (
        current_frame
        and hasattr(current_frame, "cleanup")
        and frame_name != "owner_dashboard"
    ):
        print("Cleaning up auto-refresh jobs for owner dashboard")
        current_frame.cleanup()

    if frame_name not in ["login", "register"]:
        print(
            f"Switching to {frame_name} with token: {access_token[:10] if access_token else 'None'}"
        )

    print(f"Raising frame: {frame_name}")
    frame.tkraise()


def initialize_authenticated_frames(token):
    """Initialize or reinitialize all frames that require authentication"""
    global frames

    # Recreate authenticated frames with the new token
    frames["create_shop"] = create_shop_frame(root, switch_frame, API_URL, token)
    frames["view_shop"] = view_shop_frame(root, switch_frame, API_URL, token, None)
    frames["create_product"] = create_product_frame(root, switch_frame, API_URL, token)
    frames["edit_product"] = edit_product_frame(
        root, switch_frame, API_URL, token, None
    )
    frames["view_product"] = view_product_frame(
        root, switch_frame, API_URL, token, None
    )
    frames["category"] = category_frame(root, switch_frame, API_URL, token)
    frames["dashboard"] = dashboard_frame(root, switch_frame, API_URL, token)
    frames["owner_dashboard"] = owner_dashboard_frame(
        root, switch_frame, API_URL, token
    )
    frames["owner_products"] = owner_products_frame(root, switch_frame, API_URL, token)
    frames["owner_orders"] = owner_orders_frame(root, switch_frame, API_URL, token)
    frames["user_details"] = user_details_frame(root, switch_frame, API_URL, token)
    frames["user_orders"] = user_orders_frame(root, switch_frame, API_URL, token)
    frames["user_payments"] = user_payments_frame(root, switch_frame, API_URL, token)

    # Place all authenticated frames
    for key, frame in frames.items():
        if key not in ["login", "register"]:
            frame.place(relx=0, rely=0, relwidth=1, relheight=1)


def main():
    global root, frames, user_role
    try:
        print("Starting Shopping App...")
        ctk.set_appearance_mode("dark")
        ctk.set_default_color_theme("blue")

        root = ctk.CTk()
        root.title("Shopping App")
        root.geometry("1000x800")
        print("Window created successfully")

        # Create frames
        frames = {}

        # Login and register don't need tokens
        print("Creating login and register frames...")
        login = login_frame(root, switch_frame, API_URL)
        register = register_frame(root, switch_frame, API_URL)

        # Collect all frames in a dictionary - start with unauthenticated frames
        frames = {
            "login": login,
            "register": register,
        }

        # Place login and register frames
        frames["login"].place(relx=0, rely=0, relwidth=1, relheight=1)
        frames["register"].place(relx=0, rely=0, relwidth=1, relheight=1)
        print("Login and register frames placed")

        # Create authenticated frames with empty token (they'll be recreated on login)
        print("Initializing authenticated frames...")
        initialize_authenticated_frames(access_token)
        print("All frames initialized")

        # Start with login frame
        switch_frame("login")
        print("Starting main loop...")
        root.mainloop()
    except Exception as e:
        print(f"Error starting application: {e}")
        traceback.print_exc()


if __name__ == "__main__":
    main()
